@@ -200,11 +200,15 @@ class AccumulationRenderer(nn.Module):
 class DepthRenderer(nn.Module):
     """Calculate depth along ray.
 
+    Depth Method:
+        - median: Depth is set to the distance where the accumulated weight reaches 0.5.
+        - expected: Expected depth along ray. Same procedure as rendering rgb, but with depth.
+
     Args:
-        method (str, optional): Depth calculation method.
+        method: Depth calculation method.
     """
 
-    def __init__(self, method: Literal["expected"] = "expected") -> None:
+    def __init__(self, method: Literal["median", "expected"] = "median") -> None:
         super().__init__()
         self.method = method
 
@@ -215,7 +219,7 @@ class DepthRenderer(nn.Module):
         ray_indices: Optional[TensorType["num_samples"]] = None,
         num_rays: Optional[int] = None,
     ) -> TensorType[..., 1]:
-        """Composite samples along ray and calculate disparities.
+        """Composite samples along ray and calculate depths.
 
         Args:
             weights: Weights for each sample.
@@ -227,6 +231,17 @@ class DepthRenderer(nn.Module):
             Outputs of depth values.
         """
 
+        if self.method == "median":
+            steps = (ray_samples.frustums.starts + ray_samples.frustums.ends) / 2
+
+            if ray_indices is not None and num_rays is not None:
+                raise NotImplementedError("Median depth calculation is not implemented for packed samples.")
+            cumulative_weights = torch.cumsum(weights[..., 0], dim=-1)  # [..., num_samples]
+            split = torch.ones((*weights.shape[:-2], 1), device=weights.device) * 0.5  # [..., 1]
+            median_index = torch.searchsorted(cumulative_weights, split, side="left")  # [..., 1]
+            median_index = torch.clamp(median_index, 0, steps.shape[-2] - 1)  # [..., 1]
+            median_depth = torch.gather(steps[..., 0], dim=-1, index=median_index)  # [..., 1]
+            return median_depth
         if self.method == "expected":
             eps = 1e-10
             steps = (ray_samples.frustums.starts + ray_samples.frustums.ends) / 2
