@@ -1,4 +1,4 @@
-# Copyright 2022 The Plenoptix Team. All rights reserved.
+# Copyright 2022 The Nerfstudio Team. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -29,25 +29,25 @@ from torchmetrics.functional import structural_similarity_index_measure
 from torchmetrics.image.lpip import LearnedPerceptualImagePatchSimilarity
 
 from nerfstudio.cameras.rays import RayBundle
-from nerfstudio.configs.utils import to_immutable_dict
-from nerfstudio.fields.modules.encoding import TensorVMEncoding
-from nerfstudio.fields.modules.field_heads import FieldHeadNames
-from nerfstudio.fields.nerf_field import NeRFField
-from nerfstudio.models.base import Model, VanillaModelConfig
-from nerfstudio.models.modules.ray_sampler import PDFSampler, UniformSampler
-from nerfstudio.optimizers.loss import L1Loss, MSELoss
-from nerfstudio.optimizers.optimizers import Optimizers
-from nerfstudio.renderers.renderers import (
-    AccumulationRenderer,
-    DepthRenderer,
-    RGBRenderer,
-)
-from nerfstudio.utils import colors, misc, visualization
-from nerfstudio.utils.callbacks import (
+from nerfstudio.configs.config_utils import to_immutable_dict
+from nerfstudio.engine.callbacks import (
     TrainingCallback,
     TrainingCallbackAttributes,
     TrainingCallbackLocation,
 )
+from nerfstudio.field_components.encodings import NeRFEncoding, TensorVMEncoding
+from nerfstudio.field_components.field_heads import FieldHeadNames
+from nerfstudio.fields.tensorf_field import TensoRFField
+from nerfstudio.model_components.losses import MSELoss
+from nerfstudio.model_components.ray_samplers import PDFSampler, UniformSampler
+from nerfstudio.model_components.renderers import (
+    AccumulationRenderer,
+    DepthRenderer,
+    RGBRenderer,
+)
+from nerfstudio.model_components.scene_colliders import AABBBoxCollider
+from nerfstudio.models.base_model import Model, VanillaModelConfig
+from nerfstudio.utils import colormaps, colors, misc
 
 
 @dataclass
@@ -58,12 +58,20 @@ class TensoRFModelConfig(VanillaModelConfig):
     """target class to instantiate"""
     init_resolution: int = 128
     """initial render resolution"""
-    final_resolution: int = 200
+    final_resolution: int = 300
     """final render resolution"""
-    upsampling_iters: Tuple[int, ...] = (5000, 5500, 7000)
+    upsampling_iters: Tuple[int, ...] = (2000, 3000, 4000, 5500, 7000)
     """specifies a list of iteration step numbers to perform upsampling"""
-    loss_coefficients: Dict[str, float] = to_immutable_dict({"rgb_loss_coarse": 1.0, "feature_loss": 8e-5})
+    loss_coefficients: Dict[str, float] = to_immutable_dict({"rgb_loss": 1.0})
     """Loss specific weights."""
+    num_samples: int = 256
+    """Number of samples in field evaluation"""
+    num_den_components: int = 16
+    """Number of components in density encoding"""
+    num_color_components: int = 48
+    """Number of components in color encoding"""
+    appearance_dim: int = 27
+    """Number of channels for color encoding"""
 
 
 class TensoRFModel(Model):
@@ -80,6 +88,9 @@ class TensoRFModel(Model):
     ) -> None:
         self.init_resolution = config.init_resolution
         self.upsampling_iters = config.upsampling_iters
+        self.num_den_components = config.num_den_components
+        self.num_color_components = config.num_color_components
+        self.appearance_dim = config.appearance_dim
         self.upsampling_steps = (
             np.round(
                 np.exp(
@@ -106,18 +117,21 @@ class TensoRFModel(Model):
             resolution = self.upsampling_steps.pop(0)
 
             # upsample the position and direction grids
-            # TODO(ethan): ask Brent how to get typing to work on this... the Encoding base class type
-            # in NeRFField is causing the issue
-            self.field.position_encoding.upsample_grid(resolution)
-            self.field.direction_encoding.upsample_grid(resolution)
+            self.field.density_encoding.upsample_grid(resolution)
+            self.field.color_encoding.upsample_grid(resolution)
 
-            # reinitialize the optimizer
+            # reinitialize the encodings optimizer
             optimizers_config = training_callback_attributes.optimizers.config
-            training_callback_attributes.optimizers = Optimizers(
-                optimizers_config, training_callback_attributes.pipeline.get_param_groups()
-            )
-            # TODO(ethan): do something with the learning rate
-            # we don't want to reinitialize the learning rate each time
+            enc = training_callback_attributes.pipeline.get_param_groups()["encodings"]
+            lr_init = optimizers_config["encodings"]["optimizer"].lr
+
+            training_callback_attributes.optimizers.optimizers["encodings"] = optimizers_config["encodings"][
+                "optimizer"
+            ].setup(params=enc)
+            if optimizers_config["encodings"]["scheduler"]:
+                training_callback_attributes.optimizers.schedulers["encodings"] = optimizers_config["encodings"][
+                    "scheduler"
+                ].setup(optimizer=training_callback_attributes.optimizers.optimizers["encodings"], lr_init=lr_init)
 
         callbacks = [
             TrainingCallback(
@@ -134,25 +148,33 @@ class TensoRFModel(Model):
         super().populate_modules()
 
         # setting up fields
-        position_encoding = TensorVMEncoding(
+        density_encoding = TensorVMEncoding(
             resolution=self.init_resolution,
-            num_components=96,
+            num_components=self.num_den_components,
         )
-        direction_encoding = TensorVMEncoding(
+        color_encoding = TensorVMEncoding(
             resolution=self.init_resolution,
-            num_components=96,
+            num_components=self.num_color_components,
         )
 
-        self.field = NeRFField(
-            position_encoding=position_encoding,
+        feature_encoding = NeRFEncoding(in_dim=self.appearance_dim, num_frequencies=2, min_freq_exp=0, max_freq_exp=2)
+        direction_encoding = NeRFEncoding(in_dim=3, num_frequencies=2, min_freq_exp=0, max_freq_exp=2)
+
+        self.field = TensoRFField(
+            self.scene_box.aabb,
+            feature_encoding=feature_encoding,
             direction_encoding=direction_encoding,
-            base_mlp_num_layers=2,
-            base_mlp_layer_width=128,
+            density_encoding=density_encoding,
+            color_encoding=color_encoding,
+            appearance_dim=self.appearance_dim,
+            head_mlp_num_layers=2,
+            head_mlp_layer_width=128,
+            use_sh=False,
         )
 
         # samplers
-        self.sampler_uniform = UniformSampler(num_samples=self.config.num_coarse_samples)
-        self.sampler_pdf = PDFSampler(num_samples=self.config.num_importance_samples)
+        self.sampler_uniform = UniformSampler(num_samples=self.config.num_samples, single_jitter=True)
+        self.sampler_pdf = PDFSampler(num_samples=self.config.num_samples // 2, single_jitter=True)
 
         # renderers
         self.renderer_rgb = RGBRenderer(background_color=colors.WHITE)
@@ -161,34 +183,56 @@ class TensoRFModel(Model):
 
         # losses
         self.rgb_loss = MSELoss()
-        self.feature_loss = L1Loss()
 
         # metrics
         self.psnr = PeakSignalNoiseRatio(data_range=1.0)
         self.ssim = structural_similarity_index_measure
         self.lpips = LearnedPerceptualImagePatchSimilarity()
 
+        # colliders
+        if self.config.enable_collider:
+            self.collider = AABBBoxCollider(scene_box=self.scene_box)
+
     def get_param_groups(self) -> Dict[str, List[Parameter]]:
         param_groups = {}
-        param_groups["fields"] = list(self.field.parameters())
-        param_groups["position_encoding"] = list(self.field.position_encoding.parameters())
-        param_groups["direction_encoding"] = list(self.field.direction_encoding.parameters())
+
+        param_groups["fields"] = (
+            list(self.field.mlp_head.parameters())
+            + list(self.field.B.parameters())
+            + list(self.field.field_output_rgb.parameters())
+        )
+        param_groups["encodings"] = list(self.field.color_encoding.parameters()) + list(
+            self.field.density_encoding.parameters()
+        )
+
         return param_groups
 
     def get_outputs(self, ray_bundle: RayBundle):
-
         # uniform sampling
         ray_samples_uniform = self.sampler_uniform(ray_bundle)
+        dens = self.field.get_density(ray_samples_uniform)
+        weights = ray_samples_uniform.get_weights(dens)
+        coarse_accumulation = self.renderer_accumulation(weights)
+        acc_mask = torch.where(coarse_accumulation < 0.0001, False, True).reshape(-1)
+
+        # pdf sampling
+        ray_samples_pdf = self.sampler_pdf(ray_bundle, ray_samples_uniform, weights)
+
+        # fine field:
+        field_outputs_fine = self.field.forward(ray_samples_pdf, acc_mask, colors.WHITE.to(weights.device))
+
+        weights_fine = ray_samples_pdf.get_weights(field_outputs_fine[FieldHeadNames.DENSITY])
+
+        accumulation = self.renderer_accumulation(weights_fine)
+        depth = self.renderer_depth(weights_fine, ray_samples_pdf)
 
-        # coarse field:
-        field_outputs_coarse = self.field.forward(ray_samples_uniform)
-        weights = ray_samples_uniform.get_weights(field_outputs_coarse[FieldHeadNames.DENSITY])
         rgb = self.renderer_rgb(
-            rgb=field_outputs_coarse[FieldHeadNames.RGB],
-            weights=weights,
+            rgb=field_outputs_fine[FieldHeadNames.RGB],
+            weights=weights_fine,
         )
-        accumulation = self.renderer_accumulation(weights)
-        depth = self.renderer_depth(weights, ray_samples_uniform)
+
+        rgb = torch.where(accumulation < 0, colors.WHITE.to(rgb.device), rgb)
+        accumulation = torch.clamp(accumulation, min=0)
 
         outputs = {"rgb": rgb, "accumulation": accumulation, "depth": depth}
         return outputs
@@ -197,16 +241,10 @@ class TensoRFModel(Model):
         # Scaling metrics by coefficients to create the losses.
         device = outputs["rgb"].device
         image = batch["image"].to(device)
-        assert isinstance(self.field.position_encoding, TensorVMEncoding)
 
         rgb_loss = self.rgb_loss(image, outputs["rgb"])
-        plane_coef = self.field.position_encoding.plane_coef
-        line_coef = self.field.position_encoding.line_coef
 
-        plane_feature_loss = self.feature_loss(plane_coef, torch.zeros_like(plane_coef))
-        line_feature_loss = self.feature_loss(line_coef, torch.zeros_like(line_coef))
-
-        loss_dict = {"rgb_loss": rgb_loss, "feature_loss": plane_feature_loss + line_feature_loss}
+        loss_dict = {"rgb_loss": rgb_loss}
         loss_dict = misc.scale_dict(loss_dict, self.config.loss_coefficients)
         return loss_dict
 
@@ -215,8 +253,8 @@ class TensoRFModel(Model):
     ) -> Tuple[Dict[str, float], Dict[str, torch.Tensor]]:
         image = batch["image"].to(outputs["rgb"].device)
         rgb = outputs["rgb"]
-        acc = visualization.apply_colormap(outputs["accumulation"])
-        depth = visualization.apply_depth_colormap(
+        acc = colormaps.apply_colormap(outputs["accumulation"])
+        depth = colormaps.apply_depth_colormap(
             outputs["depth"],
             accumulation=outputs["accumulation"],
             near_plane=self.config.collider_params["near_plane"],
