@@ -21,7 +21,7 @@ from typing import Type
 
 import torch
 
-from nerfstudio.data.datamanagers import VanillaDataManager
+from nerfstudio.data.datamanagers.base_datamanager import VanillaDataManager
 from nerfstudio.pipelines.base_pipeline import VanillaPipeline, VanillaPipelineConfig
 
 
