@@ -17,10 +17,11 @@ Profiler base class and functionality
 """
 from __future__ import annotations
 
-import logging
 import time
 from typing import Callable
 
+from rich.console import Console
+
 from nerfstudio.configs import base_config as cfg
 from nerfstudio.utils import comms
 from nerfstudio.utils.decorators import (
@@ -29,6 +30,8 @@ from nerfstudio.utils.decorators import (
     decorate_all,
 )
 
+CONSOLE = Console(width=120)
+
 PROFILER = []
 
 
@@ -82,7 +85,7 @@ class Profiler:
 
     def print_profile(self):
         """helper to print out the profiler stats"""
-        logging.info("Printing profiling stats, from longest to shortest duration in seconds")
+        CONSOLE.print("Printing profiling stats, from longest to shortest duration in seconds")
         sorted_keys = sorted(
             self.profiler_dict.keys(),
             key=lambda k: self.profiler_dict[k]["val"],
@@ -90,4 +93,4 @@ class Profiler:
         )
         for k in sorted_keys:
             val = f"{self.profiler_dict[k]['val']:0.4f}"
-            print(f"{k:<20}: {val:<20}")
+            CONSOLE.print(f"{k:<20}: {val:<20}")
