@@ -7,7 +7,7 @@ import tyro
 from rich.console import Console
 from rich.style import Style
 
-console = Console(width=120)
+CONSOLE = Console(width=120)
 
 LOCAL_TESTS = ["Run license checks", "Run Black", "Python Pylint", "Test with pytest"]
 
@@ -20,7 +20,7 @@ def run_command(command: str) -> None:
     """
     ret_code = subprocess.call(command, shell=True)
     if ret_code != 0:
-        console.print(f"[bold red]Error: `{command}` failed. Exiting...")
+        CONSOLE.print(f"[bold red]Error: `{command}` failed. Exiting...")
         sys.exit(1)
 
 
@@ -31,20 +31,20 @@ def main(clean_cache: bool = False):
         clean_cache: whether to clean the cache before building docs.
     """
 
-    console.print("[green]Adding notebook documentation metadata")
+    CONSOLE.print("[green]Adding notebook documentation metadata")
     run_command("python scripts/docs/add_nb_tags.py")
 
     # Add checks for building documentation
-    console.print("[green]Building Documentation")
+    CONSOLE.print("[green]Building Documentation")
     if clean_cache:
         run_command("cd docs/; make clean; make html SPHINXOPTS='-W;'")
     else:
         run_command("cd docs/; make html SPHINXOPTS='-W;'")
 
-    console.line()
-    console.rule(characters="=", style=Style(color="green"))
-    console.print("[bold green]Done")
-    console.rule(characters="=", style=Style(color="green"))
+    CONSOLE.line()
+    CONSOLE.rule(characters="=", style=Style(color="green"))
+    CONSOLE.print("[bold green]Done")
+    CONSOLE.rule(characters="=", style=Style(color="green"))
 
 
 if __name__ == "__main__":
